package MarshalledObjectPersistenz;

import java.io.*;
import java.rmi.*;
import java.rmi.activation.*;
import java.util.Vector;

public class MeinePersistenteKlasse extends Activatable
    implements RemoteInterface {

    private Vector  transaktionen;
    private File    speicher;

    // Konstruktor fr Aktivierung und Export
    // Dieser wird von der Methode ActivationInstantiator.newInstance
    // in der Aktivierung aufgerufen, um das Objekt zu konstruieren.
    //
    public MeinePersistenteKlasse(ActivationID id, MarshalledObject data)
	throws RemoteException, ClassNotFoundException, java.io.IOException {

	super(id, 0);
        System.out.println("Persistenz: Aktivierungs- und Export-Konstruktor");
        // Extrahieren  des File Objekts aus dem MarshalledObject
        // aus dem Konstruktor
	//
        System.out.println("Persistenz: Aktivierungs- und Export-Konstruktor-Bestimmen derDatei");
	speicher = (File)data.get();

	if (speicher.exists()) {
	    // falls das MarshalledObject existiert:
            // restore des Objektzustands
	    //
            System.out.println("Persistenz: Konstruktor - Serialisiertes Objekt wird gelesen");
            System.out.println("Persistenz: Konstruktor - restoreState()");
	    this.restoreState();
	} else {
            System.out.println("Persistenz: Konstruktor - Init Transaktionsvektors");
	    transaktionen = new Vector(1,1);
	    transaktionen.addElement("Persistenz: Init Transaktionsvektors");
	}
    }

    // Definion der im RemoteInterface deklarierten Methoden
    //
    public Vector rufDenServer(Vector v) throws RemoteException {

        System.out.println("Persistenz: rufDenServer()");
	int limit = v.size();
	for (int i = 0; i < limit; i++) {
	    transaktionen.addElement(v.elementAt(i));
	}

	// Speichern der Objektdaten in der Datei 'file'
	//
        System.out.println("Persistenz: rufDenServer() - Speichern des Zustands");
	this.saveState();
	return transaktionen;
    }

    public Vector liesTransaktionen() {
        System.out.println("Persistenz: liesTransaktionen");
	return transaktionen;
    }


    // Falls das MarshalledObject, welches dem Konstruktor bergeben wurde
    // eine Datei ist, wird es benutzt um die Transaktionsdaten zu
    // rekonstruieren.
    //
    private void restoreState() throws IOException, ClassNotFoundException {
        System.out.println("Persistenz: Lesen des Serialisierten Objekts");
        System.out.println("Persistenz: restoreState()");
	File f = speicher;
	FileInputStream fis = new FileInputStream(f);
        ObjectInputStream ois = new ObjectInputStream(fis);
        transaktionen = (Vector)ois.readObject();
        ois.close();
    }

    private void saveState() {
        System.out.println("Persistenz: saveState()");
        FileOutputStream fos = null;
        ObjectOutputStream oos = null;

	try {
	    File f  = speicher;
		try {
	            fos = new FileOutputStream(f);
		} catch (IOException e1) {
		    e1.printStackTrace();
	            throw new RuntimeException("Persistenz: FileOutputStream konnte nicht kreiert werden");
		}
		try {
	            oos = new ObjectOutputStream(fos);
		} catch (IOException e2) {
	            throw new RuntimeException("Persistenz: ObjectOutputStream konnte nicht kreiert werden");
		}
		try {
	            oos.writeObject(liesTransaktionen());
		} catch (IOException e3) {
	            throw new RuntimeException("Persistenz: Fehler beim Schreiben des Vectors");
		}
		try {
	            oos.close();
		} catch (IOException e3) {
	            throw new RuntimeException("Persistenz: Fehler beim Schliessen des Stream");
		}

	} catch (SecurityException e4) {
	    throw new RuntimeException("Persistenz: Security Problem");
	} catch (Exception e) {
	    throw new RuntimeException("Persistenz: Fehler beim Speichern des Datenvektors");
	}
    }
}
