package HelloWorld;

import java.io.*;
import java.net.*;

class CompressionInputStream extends FilterInputStream
    implements CompressionConstants
{
    /*
     * Konstruktor: ruft den Oberklassen-Konstruktor auf
     */
    public CompressionInputStream(InputStream in) {
        super(in);
    }

    /*
     * Buffer : entpackte 6-Bit Codes
     * der letzten 32 gelesenen Bits.
     */
    int buf[] = new int[5];

    /*
     * Position des nchsten Codes im Buffer (5 : Ende).
     */
    int bufPos = 5;

    /*
     * Lesen der Format Code und Dekompression der Zeichen.
     */

    public int read() throws IOException {
        try {
            int code;

            // Lesen und ignorieren der leeren (NOP's)
            do {
	      code = readCode();
	    } while (code == NOP);

            if (code >= BASE) {
                // Index des Zeichens in der codeTable
                // falls der Code im Bereich liegt.
                return codeTable.charAt(code - BASE);
            } else if (code == RAW) {
                // lesen der unteren 4 Bits und der oberen 4 Bits,
                // Rckgabe des rekonstruierten Zeichens
                int high = readCode();
                int low = readCode();
                return (high << 4) | low;
            } else
                throw new IOException("unbekannter Kompressionscode: " + code);
        } catch (EOFException e) {
            // EOF Code
            return -1;
        }
    }

    /*
     * Method liest bis zu len Bytes vom Input Stream.
     * Abbruch, falls read vor dem Lesen von len Bytes blockiert.
     */
    public int read(byte b[], int off, int len) throws IOException {

	if (len <= 0) {
	    return 0;
	}

	int c = read();
	if (c == -1) {
	    return -1;
	}
	b[off] = (byte)c;

	int i = 1;
        // Versuche len Bytes zu lesen oder bis
        // keine Bytes ohne Blockung gelesen werden knnen.
       	try {
		for (; (i < len) && (in.available() > 0); i++) {
		c = read();
		if (c == -1) {
		    break;
		}
		if (b != null) {
		    b[off + i] = (byte)c;
		}
	    }
	} catch (IOException ee) {
	}
	return i;
    }

    /*
     * Falls in buf nichts mehr zu entschlsseln ist,
     * lies die nchsten vier Bytes.
     * Speichere jede Gruppe von 6 Bits in einem buf Element.
     */
    private int readCode() throws IOException {
        // Falls alle Daten in buf gelesen wurden,
        // (bufPos == 5) lies weitere 4 Bytes.
        if (bufPos == 5) {
            int b1 = in.read();
            int b2 = in.read();
            int b3 = in.read();
            int b4 = in.read();

            // EOF Test
            if ((b1 | b2 | b3 | b4) < 0) {
                throw new EOFException();
            }
            // 6 Bits an ein Element in
            // buf zuweisen
            int pack = (b1 << 24) | (b2 << 16) | (b3 << 8) | b4;
            buf[0] = (pack >>> 24) & 0x3F;
            buf[1] = (pack >>> 18) & 0x3F;
            buf[2] = (pack >>> 12) & 0x3F;
            buf[3] = (pack >>>  6) & 0x3F;
            buf[4] = (pack >>>  0) & 0x3F;
            bufPos = 0;
        }
        return buf[bufPos++];
    }
}
