package Komprimierung;

import java.io.*;

class CompressionOutputStream extends FilterOutputStream
    implements CompressionConstants
{

    /*
     * Konstruktor ruft den Konstruktor der Oberklasse auf.
     */
    public CompressionOutputStream(OutputStream out) {
        super(out);
    }

    /*
     * Buffer der 6-Bit Codes in die das 32-Bit Wort gepackt wird.
     * Fnf 6-Bit Codes passen in 4 Worte.
     */
    int buf[] = new int[5];

    /*
     * Index der gltigen Codes in buf.
     */
    int bufPos = 0;

    /*
     * Diese Methode schreibt ein Byte in den Socket Stream.
     */
    public void write(int b) throws IOException {
        //
        b &= 0xFF;

        // Look Position in der Code Tabelle codeTable, um die Verschlsselung
        // zu erhalten.
        int pos = codeTable.indexOf((char)b);

        if (pos != -1){
            // Falls pos in the codeTable ist, schreib BASE + pos in buf.
            // BASE zeigt, dass pos Zeichen in
            // der codeTable enthalten ist und einen Code zwischen
            //  2 und 63
            // inklusive besitzt.
            // RAW ist gleich 1.

            writeCode(BASE + pos);
        } else {
            // schreib RAW in buf:
            // das Zeichen wird in 12 Bits gesendet.
            writeCode(RAW);

            // schreib 4 Bits von b in buf.
            writeCode(b >> 4);

            // b modifizieren, so dass nur die ersten 4 Bits enthalten sind,
            // schreib die ersten 4 Bits von b in buf.
            writeCode(b & 0xF);
        }
    }

    /*
     * Diese Methode schreibt bis zu len Bytes in den Socket Stream.
     */
    public void write(byte b[], int off, int len) throws IOException {
        /*
         * Diese Implementation ist recht ineffizient, da sie
         * die write Methode fr jedes Byte im Array aufrufen muss.
         */
        for (int i = 0; i < len; i++)
            write(b[off + i]);
    }


   /*
    * Rcksetzen des Buffers.
    */
  public void flush() throws IOException {
    while (bufPos > 0)
      writeCode(NOP);
  }

    /*
     * Diese Methode schreibt die Daten in den Output Stream nachdem
     * die Daten aller 5 Bytes in buf in ein Wort geschrieben sind.
     */
    private void writeCode(int c) throws IOException {
        buf[bufPos++] = c;
        if (bufPos == 5) {      // 5 Codes
            int pack = (buf[0] << 24) | (buf[1] << 18) | (buf[2] << 12) |
                       (buf[3] << 6) | buf[4];
            out.write((pack >>> 24) & 0xFF);
            out.write((pack >>> 16) & 0xFF);
            out.write((pack >>> 8)  & 0xFF);
            out.write((pack >>> 0)  & 0xFF);
            bufPos = 0;
        }
    }
}
